/* LBINSTDJ.C - DJGPP installer, v0.2   */
/* Copyright (C) 1999 Laurynas Biveinis */

/* Uncomment next line if you want "is FPU present" check and autoexe update*/
/* #define INCLUDE_FPU */

#include <sys/stat.h>
#include <conio.h>
#include <ctype.h>
#include <dos.h>
#include <dpmi.h>
#include <go32.h>
#include <process.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#define DEFAULT_DJGPP   "C:\\DJGPP"

#define UNZIPPER        "unzip"

#define INTRO           "LBINSTDJ v0.2, DJGPP installer"

#define PLAIN_DOS_WARN  "Note - you are running DOS ver. 7 but not under Windows. Registry update\n"\
                        "is useful and long file names are supported if you run LBInstDJ in Windows\n"\
                        "DOS box *only*. So please start Windows and run LBInstDJ there.\n"

#define GET_DJGPP_PATH  "Enter path to downloaded DJGPP zips: "

#define GET_ZIP_PATH    "Enter directory for DJGPP, default is %s: "

#define DEV_IS_WRONG    "You cannot install DJGPP to /dev directory. See the FAQ for details.\n"

      /* Note - /dev changed to \DEV by change_slashes() and strupr() */
#define DEV_DIR         "\\DEV"

#define DJ_CREATE_ERROR "Error creating %s\n"

#define UPDATE_AUTOEXE  "Updating c:\\autoexec.bat..."

#define RESTART         "You must restart computer now"

#ifdef INCLUDE_FPU
#define ENV_387         "SET 387=N"

#define ENV_EMU         "SET EMU387=%s\\BIN\\emu387.dxe"

#define ASK_FPU         "FPU not found, do you want to update your autoexec.bat?\n"\
                        "(don't do that if you actually have the FPU) Y/N:"
#endif /* #ifdef INCLUDE_FPU */

#define PATH_ENV_DJGPP  "\n"\
                        "REM Following lines were added by LBINSTDJ, the DJGPP installer\n"\
                        "SET PATH=%s\\BIN;%%PATH%%\n"\
                        "SET DJGPP=%s\\djgpp.env\n"

#define ASK_REG_UPDATE  "If you want to use DJGPP in Windows 9x and in plain DOS, it is recommended\n"\
                        "that you let to update Windows Registry and disable numeric tails in short\n"\
                        "file names (see FAQ for details). After this restart Windows and run LBInstDJ\n"\
                        "again. If you have already done this or don't want to update registry,\n"\
                        "choose N, otherwise Y. Do you want to update registry? (Y/N) "

#define REGISTRY_UPDATE "\nUpdating Windows Registry..."

#define REG_FILE        "NoNumTl.reg"

     /* Adds "\" to directory name if there is no "/" nor "\\" at the end */
void add_ending_backslash(char *directory);

     /* Changes all "/" to "\" */
void change_slashes(char *directory);

     /* Read from console characters until 'Y', 'y', 'N', 'n'               */
     /* Result is uppercased                                                */
char get_choice(void);

#ifdef INCLUDE_FPU
     /* Tries to detect FPU - checks bit in BDA and enviroment variable 387 */
int  is_fpu(void);
#endif

     /* Checks if program is running in DOS box under Windows95 / Windows98 */
     /* I assume that prg is in box if dos major ver is 7 and get current   */
     /* directory for current drive function LFN version does not fail      */
int is_win9x_dos_box(void);

     /* Checks if directory isn't LFN */
/* int  is_valid_sfn(char *directory); */

int main(void)
{
   FILE        *autoexec_bat;
   char         djgpp_path[256];
   char         zip_path[256];
   char         choice;
   int          compare_result;

   puts(INTRO);

   /* Optionally update Windows Registry */
   if (_osmajor == 7)
   {
      printf(ASK_REG_UPDATE);
      choice = get_choice();
      if (choice == 'Y')
      {
         puts(REGISTRY_UPDATE);
         spawnlp(P_WAIT, "regedit", "regedit", REG_FILE, NULL);
         if (!is_win9x_dos_box())
            puts(PLAIN_DOS_WARN);
         else
            puts(RESTART);
         exit(EXIT_SUCCESS);
      }
   }

   /* Get path where zips are stored */
   printf(GET_DJGPP_PATH);
   gets(zip_path);
   strupr(zip_path);
   change_slashes(zip_path);
   add_ending_backslash(zip_path);

   /* Get directory, where DJGPP install to */
   do
   {
      printf(GET_ZIP_PATH, DEFAULT_DJGPP);
      gets(djgpp_path);
      change_slashes(djgpp_path);
      strupr(djgpp_path);
      compare_result = strnicmp(djgpp_path, DEV_DIR, strlen(DEV_DIR));
      if (!compare_result)
         puts(DEV_IS_WRONG);
   }
   while (!compare_result);
   /* If empty string, change to C:\DJGPP */
   if (!strlen(djgpp_path))
      strcpy(djgpp_path, DEFAULT_DJGPP);

   /* Check if directory exists, create if needed */
   if (access(djgpp_path, F_OK | D_OK))
      if (mkdir(djgpp_path, S_IWUSR))
      {
         fprintf(stderr, DJ_CREATE_ERROR, djgpp_path);
         perror("");
         exit(EXIT_FAILURE);
      }

   /* Extracting zips */
   strcat(zip_path, "*.zip");
   spawnlp(P_WAIT, UNZIPPER, "-o", zip_path, "-d", djgpp_path, NULL);

   /* Updating autoexec.bat */
   puts(UPDATE_AUTOEXE);
   system("copy c:\\autoexec.bat c:\\autoexec.lbi");
   autoexec_bat = fopen("c:\\autoexec.bat", "at");
   if (!autoexec_bat)
   {
      perror("C:\\autoexec.bat");
      exit(EXIT_FAILURE);
   }
   fprintf(autoexec_bat, PATH_ENV_DJGPP, djgpp_path, djgpp_path);
#ifdef INCLUDE_FPU
   if (!is_fpu())
   {
      printf(ASK_FPU);
      choice = get_choice();
      if (choice == 'Y')
      {
         fprintf(autoexec_bat, ENV_EMU, djgpp_path);
         if (!getenv("387"))
             fputs(ENV_387, autoexec_bat);
      }
   }
#endif /* #ifdef INCLUDE_FPU */
   fclose(autoexec_bat);
   puts(RESTART);
   return 0;
}

void add_ending_backslash(char *directory)
{
   size_t  dir_length =  strlen(directory);
   char   *last_char  = &directory[dir_length - 1];
   if ((*last_char != '\\') && (*last_char != '/') && dir_length)
      strcat(directory, "\\");
}

/*
int is_valid_sfn(char *directory)
{
   __dpmi_regs registers;
   int         selector;
   int         segment = __dpmi_allocate_dos_memory(8, &selector);
   char        sfn[128];
   dosmemput(directory, strlen(directory) + 1, segment << 4);
   registers.x.ax = 0x7160;
   registers.x.cx = 0x8001;
   registers.x.ds = segment;
   registers.x.si = 0x0000;
   registers.x.es = __tb >> 4;
   registers.x.di = __tb & 0x0F;
   registers.x.flags = 0x0001;
   __dpmi_int(0x21, &registers); */
   /* If CF flag still set */
/*   if (registers.x.flags & 0x0001)
      return 1;
   dosmemget(__tb, 128, sfn);
   return !strcmp(directory, sfn);
}
*/
void change_slashes(char *directory)
{
   char *current_ptr = directory;
   while (*current_ptr)
   {
      if (*current_ptr == '/')
         *current_ptr = '\\';
      current_ptr++;
   }
}

#ifdef INCLUDE_FPU
int is_fpu(void)
{
   unsigned char eq_byte;
   char *env387 = getenv("387");
   dosmemget(0x0410, 1, &eq_byte);
   return ((eq_byte & 0x02) || !stricmp(env387, "Y")) && stricmp(env387, "N");
}
#endif

int is_win9x_dos_box(void)
{
   __dpmi_regs registers;
   registers.x.ax = 0x7147;
   registers.h.dl = 0x00;
/*   registers.x.flags = 0x0001; */  /* Set CF */
   registers.x.ds = __tb >> 4;
   registers.x.si = __tb & 0x0F;
   __dpmi_int(0x21, &registers);
   return (_osmajor == 7) && (registers.x.ax != 0x7100);
}

char get_choice(void)
{
   char choice;
   do
   {
      choice = getch();
      choice = toupper(choice);
   }
   while ((choice != 'Y') && (choice != 'N'));
   putchar('\n');
   return choice;
}
